from mpl_toolkits.mplot3d.art3d import Poly3DCollection

from matplotlib import colors as mcolors

import numpy as np


"""
:class: 'Tetrahedro'. Clase para el dibujo de Tetrahedros de a pares (uno 'Up' y uno
'Down') en la dirección [111], a partir del centro del 'Up' y el lado 'L' del cubo en el
que se inscribe. Lo armo dibujando las cuatro caras triangulares de cada Tetrahedro. 
También se puede pasar como argumento 'N', que es un número relacionado a la intensidad
del sombreado en las caras (cuánto mayor es N, más transparentes serán las caras).
"""


class Tetrahedro:

    # Inicializo
    def __init__(self, centro, L, N=1):
        self.centro = centro
        self.L = L

        self.calcula_vertices()
        self.calcula_caras()
        self.dibuja_caras(N)

    # Método para el cálulo de los vértices. Se corresponde a sumarle a 'centro' las
    # posiciones de los vértices del Tetrahedro 'Up' respecto a su centro.
    def calcula_vertices(self):
        self.vertices = self.centro + self.L / 2 * np.array(
            [[1, 1, 1], [1, -1, -1], [-1, -1, 1], [-1, 1, -1]]
        )

    # Método para encontrar los vértices de cada cara triangular del Tetrahedro.
    def calcula_caras(self):

        self.vert_up = []  # Vértices de las caras del Tetrahedro Up.
        self.vert_down = []  # Vértices de las caras del Tetrahedro Down.

        # Con este loop se consideran todas las maneras de tomar de a tres puntos entre
        # los cuatro vértices del Tetrahedro, o sea, los vértices de cada cara triangular.
        for cara in [(0, 1, 2), (0, 1, 3), (0, 2, 3), (1, 2, 3)]:
            # Las tres posiciones x de cada vértice de la cara triangular.
            vert_x = self.vertices[cara, 0]
            # Las tres posiciones y de cada vértice de la cara triangular.
            vert_y = self.vertices[cara, 1]
            # Las tres posiciones z de cada vértice de la cara triangular.
            vert_z = self.vertices[cara, 2]

            self.vert_up.append(
                [[np.array(vert) for vert in zip(vert_x, vert_y, vert_z)]]
            )

            # Las caras del Tetrahedro down se arman invirtiendo los valores de los
            # vértices y trasladándolos.
            self.vert_down.append(
                [
                    [
                        np.array(vert)
                        for vert in zip(
                            -np.array(vert_x) + 2 * self.vertices[0, 0],
                            -np.array(vert_y) + 2 * self.vertices[0, 1],
                            -np.array(vert_z) + 2 * self.vertices[0, 2],
                        )
                    ]
                ]
            )

    # Método par dibujar las caras de ambos Tetrahedros a partir de los vértices
    # de sus caras.
    def dibuja_caras(self, N):

        self.caras = []

        for i, (cara_up, cara_down) in enumerate(zip(self.vert_up, self.vert_down)):
            self.caras.append(
                Poly3DCollection(
                    cara_up,
                    facecolors=mcolors.to_rgba(
                        "mediumpurple", alpha=(0.4 - i * 0.08) / np.cbrt(N)
                    ),
                    edgecolors=mcolors.to_rgba("gray", alpha=0.2 / np.cbrt(N)),
                )
            )

            self.caras.append(
                Poly3DCollection(
                    cara_down,
                    facecolors=mcolors.to_rgba(
                        "lightskyblue", alpha=(0.4 - i * 0.08) / np.cbrt(N)
                    ),
                    edgecolors=mcolors.to_rgba("gray", alpha=0.2 / np.cbrt(N)),
                )
            )


if __name__ == "__main__":

    import matplotlib.pyplot as plt

    fig = plt.figure()
    ax = fig.add_subplot(111, projection="3d")

    t = Tetrahedro([0, 0, 0], 0.5)

    for cara in t.caras:
        ax.add_collection3d(cara)

    ax.scatter(*np.hsplit(t.vertices, 3), s=0)

    # ax.set_aspect("equal")
    ax.set_box_aspect((1, 1, 1))
    ax.view_init(20, -75)

    plt.subplots_adjust(left=0, bottom=0, right=1, top=1)
    plt.show()



import numpy as np


"""
:class: 'Cilindro'. Clase para calcular las coordenadas cartesianas de un Cilindro a lo
largo del eje 'eje' centrado en 'pivot', de radio 'radio' y largo 'largo'.
"""


class Cilindro:

    # Inicializo
    def __init__(self, pivot, eje, radio, largo):
        self.pivot = np.array(pivot)
        self.eje = np.array(eje) / np.linalg.norm(eje)  # Vector unitario
        self.radio = radio
        self.largo = largo

        self.base = (
            self.pivot - self.largo / 2 * self.eje
        )  # Coordenadas del centro de la base del cilindro
        self.tapa = (
            self.pivot + self.largo / 2 * self.eje
        )  # Coordenadas del centro de la tapa del cilindro

        self.calcula_coordenadas()

    # Método para el cálculo de las coordenadas cartesianas.
    def calcula_coordenadas(self):

        # Armo dos vectores unitarios perpendiculares entre sí y al eje del cilindro,
        # para que hagan las veces de ejes 'x' e 'y' sobre el que construir el cilindro.
        # Primero busco un vector en una dirección distinta al eje. Luego, con éste y el
        # eje, armo un vector unitario n1 perpendicular al eje. Finalmente, con eje y n1
        # armo un vector unitario n2 perpendicular a ambos.
        not_eje = np.array([1, 0, 0])
        if (self.eje == not_eje).all():
            not_eje = np.array([0, 1, 0])

        _n1 = np.cross(self.eje, not_eje)
        _n1 = _n1 / np.linalg.norm(_n1)

        _n2 = np.cross(self.eje, _n1)

        # Variables paramétricas y grid.
        _theta = np.linspace(0, 2 * np.pi, 100)
        _t = np.linspace(0, 1, 500)

        _theta, _t = np.meshgrid(_theta, _t)

        x, y, z = [
            self.base[i]
            + _t * self.largo * self.eje[i]
            + self.radio * np.sin(_theta) * _n1[i]
            + self.radio * np.cos(_theta) * _n2[i]
            for i in [0, 1, 2]
        ]

        self.coordenadas = zip(np.flipud(x), np.flipud(y), np.flipud(z))


if __name__ == "__main__":

    import matplotlib.pyplot as plt

    fig = plt.figure()
    ax = fig.add_subplot(111, projection="3d")

    c = Cilindro([1, 1, 1], [2, 5, 7], 2, 10)

    x, y, z = zip(*c.coordenadas)
    ax.plot_surface(np.array(x), np.array(y), np.array(z), color="r")

    # ax.set_aspect("equal")
    ax.set_box_aspect((1, 1, 1))
    ax.view_init(20, -75)

    plt.subplots_adjust(left=0, bottom=0, right=1, top=1)
    plt.show()



#!/usr/bin/env python3

"""
Script para plotear configuraciones de red Pirocloro.

Dependiendo de los parámetros ingresados:
* '': dibuja solamente la red pirocloro y agrega detalles como el nombres a los ejes.
* '+z': dibuja la configuración spin ice +z.
* 'ms': dibuja la configuración de saturación con el campo en [111], con monopolos 
simples positivos en todos los Tetrahedros Up.
* 'md': dibuja la configuración con monopolos dobles positivos en todos los Tetrahedros Up.
* Si se pasa el nombre de un archivo, se obtienen de él los datos para dibujar la 
configuración. Si no se pasa más que el nombre, se supone que es un output de uno de 
mis programas y se obtienen del mismo, además de los valores de spin, sus posiciones. Si
se pasa 'ch' como segundo argumento, significa que es un archivo de Chufo: se extraen de
él los valores de spin y se los ubica en los vértices de los tetrahedros. Si en cambio 
se pasa un valor de columna como segundo argumento, significa entonces que se trata de
un archivo de otra persona (que no debe tener header): se toman de esa columna los 
valores de spin y se los ubica en los vértices de los tetrahedros.
"""

# General setup
import sys
import os.path
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt

from pyrodraw import Sistema


# Figure and axes
fig = plt.figure()
ax = fig.add_subplot(111, projection="3d")


# System construction
if len(sys.argv) > 3:
    sys.exit("\n***ERROR: Too many arguments***\n")


elif len(sys.argv) == 1:
    spin_values = np.array([0, 0, 0, 0] * 256)

    # Labels
    ax.set_xlabel("x", fontfamily="serif", fontsize=20)
    ax.set_ylabel("y", fontfamily="serif", fontsize=20)
    ax.set_zlabel("z", fontfamily="serif", fontsize=20)

    ax.xaxis.labelpad = -10
    ax.yaxis.labelpad = -10
    ax.zaxis.labelpad = -10

    # Ingreso de parámetros
    L = input("\n*Size 'LxLyLz' to draw [default: 111]: ")
    L = np.array([int(i) for i in L]) if L else np.ones(3, np.int)

    numeros = input("\n*¿Enumerate sites? (y/n) [default: no]: ")
    plot_numeros = True if numeros == "y" else False

    # Construyo y ploteo el sistema.
    print("\n\n***Wait while the plot is built***\n")
    s = Sistema(spin_values, L=L)
    s.plotear(ax, plot_numeros=plot_numeros)


else:
    posiciones = np.array([])

    filename = sys.argv[1]

    if filename == "+z":
        spin_values = np.array([1, -1, 1, -1] * 256)

    elif filename == "ms":
        spin_values = np.array([1, -1, -1, -1] * 256)

    elif filename == "md":
        spin_values = np.array([-1, -1, -1, -1] * 256)

    else:
        if not os.path.isfile(filename):
            sys.exit("\n***ERROR: Not valid option or file not found***\n")

        if len(sys.argv) == 2:  # Archivos míos
            df = pd.read_csv(
                filename, sep=r"\s+", header=None, skiprows=1, float_precision="high"
            )

            spin_values = df.iloc[:, 4]

            pos = input("\n*¿Positions from file? (y/n) [default: no]: ")
            if pos == "y":
                posiciones = df.iloc[:, 1:4]

        elif len(sys.argv) == 3:

            if sys.argv[2] == "ch":  # Archivos de Chufo

                with open(filename, "rt") as f:
                    # Auxiliary list
                    spin_values = []

                    # Skip header
                    for _ in range(5):
                        next(f)

                    # Extract spin values
                    for line in f:
                        row = line.split()
                        spin_values.extend([int(s) for s in row[4:8]])

                spin_values = np.array(spin_values)

            else:  # Archivos de otros
                df = pd.read_csv(
                    filename, sep=r"\s+", header=None, float_precision="high"
                )

                spin_values = df.iloc[:, int(sys.argv[2]) - 1]

    # Ingreso de parámetros y opciones.
    L = input("\n*Size 'LxLyLz' to draw [default: 111]: ")
    L = np.array([int(i) for i in L]) if L else np.ones(3, np.int)

    inicial = input("\n*Position 'xyz' of the init cell [default: 000]: ")
    inicial = np.array([int(i) for i in inicial]) if inicial else np.zeros(3, np.int)

    field = input("\n*Field direction 'BxByBz' [default: no arrow]: ")
    field = np.array([int(i) for i in field]) if field else np.zeros(3, np.int)

    nice = input('\n*Nice spins and monopoles? (y/n) [default: no]: ')
    plot_flechas, plot_monopolos = (True, True) if nice == "y" else (False, False)

    # Construyo y ploteo el sistema.
    print("\n\n***Wait while the plot is built***\n")
    s = Sistema(spin_values, posiciones, L, inicial, field)
    s.plotear(ax, plot_flechas, plot_monopolos)


# Show
ax.view_init(20, -75)
# ax.view_init(0,270)
# ax.view_init(0,45)

# plt.savefig(filename[:-4]+'.pdf', transparent=True, bbox_inches='tight')
# plt.savefig(filename[:-4]+'.png', transparent=True, bbox_inches='tight', dpi=500)

plt.subplots_adjust(left=0, bottom=0, right=1, top=1)
plt.show()



import numpy as np
import sys

from pyrodraw import CeldaUnidad

from pyrodraw.blocks import Paralelepipedo
from pyrodraw.blocks import Flecha

"""
:class: 'Sistema'. Clase para construir y plotear todas las celdas que 
se desean del Sistema, con los valores de spin 'spin_values' y sus 
'posiciones' (si la lista está vacía se calculará), de tamaño Lx, Ly, Lz
en 'L', comenzando por la celda 'inicial'. Además es posible pasar un 
vector de campo en 'field', con el que se dibuja un flecha en esa 
dirección. Se inicializa también con 'flechas' y 'monopolos' que 
determinan qué clase de flechas y esferas se van a dibujar: True las 
lindas pero pesadas, False las feas pero livianas. El último argumento
de inicialización es 'numeros' que indica si escribir o no el número de
cada spin.
"""


class Sistema:

    # Inicializo
    def __init__(
        self,
        spin_values,
        posiciones=np.array([]),
        L=np.ones(3, np.int),
        inicial=np.zeros(3, np.int),
        field=np.zeros(3, np.int),
    ):
        self.N = len(spin_values)

        self.L = round(
            (self.N / 16) ** (1 / 3)
        )  # L del sistema completo. Se supone cúbico.

        if (
            np.all(inicial < self.L)
            and np.all(L > 0)
            and np.all(L <= self.L)
            and np.all(inicial + L <= self.L)
        ):
            self.ix = inicial[0]
            self.iy = inicial[1]
            self.iz = inicial[2]

            self.Lx = L[0]
            self.Ly = L[1]
            self.Lz = L[2]
            self.N_cells = np.prod(L)

        else:
            sys.exit(
                "\n***ERROR: Las celda/s requerida/s no se corresponde/n con la cantidad de datos ingresados***\n"
            )

        self.spin_values = np.array(spin_values)

        if posiciones.size == 0:
            self.posiciones = self.r0(self.N)
        else:
            self.posiciones = posiciones.values

        self.field = (
            field / np.linalg.norm(field) if np.linalg.norm(field) > 0 else field
        )

        # Genero la celdas y el paralelepipedo exterior para los bordes.
        self.celdas = [
            CeldaUnidad([i, j, k], self.posiciones, self.spin_values, self.N_cells)
            for j in range(self.iy, self.iy + self.Ly)
            for k in range(self.iz, self.iz + self.Lz)
            for i in range(self.ix, self.ix + self.Lx)
        ]

        self.paralelepipedo = Paralelepipedo(
            np.array([self.Lx, self.Ly, self.Lz]) * np.sqrt(8),
            np.array([self.ix, self.iy, self.iz]) * np.sqrt(8),
        )

    # Método para determinar posiciones de equilibrio (vértices de los tetrahedros).
    def r0(self, N):

        r0 = []

        for i in range(1, N, 16):
            Tetrahedron = []

            # Tetrahedros. Determino primero las posiciones de los centros de los cuatro tetrahedros up.
            Tetrahedron.append(
                np.array(
                    [
                        int(i / 16) % self.L,
                        int((i / 16) / self.L) % self.L,
                        int((i / 16) / (self.L * self.L)),
                    ]
                )
            )
            Tetrahedron.append(Tetrahedron[0] + [0.5, 0.5, 0.0])
            Tetrahedron.append(Tetrahedron[0] + [0.0, 0.5, 0.5])
            Tetrahedron.append(Tetrahedron[0] + [0.5, 0.0, 0.5])

            # Átomos. Posiciono los átomos en los vértices de cada tetrahedro.
            for j in range(4):
                r0.append(Tetrahedron[j] + [0.125, 0.125, 0.125])
                r0.append(Tetrahedron[j] + [0.125, -0.125, -0.125])
                r0.append(Tetrahedron[j] + [-0.125, -0.125, 0.125])
                r0.append(Tetrahedron[j] + [-0.125, 0.125, -0.125])

        r0 /= np.sqrt(2) / 4

        return r0

    # Método para plotear todos los componentes del sistema. Primero las celdas, después el borde del dibujo y, si corresponde, la flecha que indica la dirección del campo.
    def plotear(self, ax, plot_flechas=False, plot_monopolos=False, plot_numeros=False):

        # Celdas
        for celda in self.celdas:

            # Cubo
            ax.add_collection3d(celda.cubo.caras)

            # Tetrahedros
            for tetrahedro in celda.tetrahedros:

                for cara in tetrahedro.caras:
                    ax.add_collection3d(cara)

            # Spines
            for j, spines in enumerate(celda.spines):

                # Flechas
                if plot_flechas:

                    for flecha, color in zip(spines.flechas, spines.colores):
                        x, y, z = zip(*flecha.coordenadas)
                        ax.plot_surface(
                            np.array(x), np.array(y), np.array(z), color=color
                        )

                else:

                    spines.colores = np.concatenate(
                        (spines.colores, np.repeat(spines.colores, 2, axis=0))
                    )
                    ax.quiver(
                        *np.hsplit(spines.posiciones, 3),
                        *np.hsplit(spines.vectores, 3),
                        length=0.5,
                        arrow_length_ratio=0.5,
                        pivot="middle",
                        normalize=True,
                        capstyle="round",
                        colors=spines.colores,
                        lw=2 / np.cbrt(self.N_cells)
                    )

                # Puntos donde hay Spines que valen 0
                for i, spin in enumerate(spines.s1234):

                    if spin == 0:
                        ax.scatter(*np.hsplit(spines.posiciones[i], 3), s=50, c="C1")

                # Números
                if plot_numeros:

                    for i, pos in enumerate(spines.posiciones):
                        ax.text(
                            *pos + [0.05, 0, -0.2],
                            str(celda.spin_inicial + j * 4 + i + 1),
                            fontfamily="serif",
                            fontsize=12
                        )

            # Monopolos
            for monopolo in celda.monopolos:

                if plot_monopolos:

                    if monopolo.radio != 0:
                        x, y, z = zip(*monopolo.coordenadas)
                        ax.plot_surface(
                            np.array(x), np.array(y), np.array(z), color=monopolo.color
                        )

                else:

                    ax.scatter(
                        *monopolo.centro,
                        s=monopolo.radio * 2800 / max(self.Lx, self.Ly, self.Lz),
                        color=monopolo.color
                    )

        # Bordes
        ax.add_collection3d(self.paralelepipedo.caras)

        # Flecha de dirección del Campo
        if plot_flechas and not np.all(self.field == 0):

            field_direction = Flecha(
                [
                    (self.ix + self.Lx + 0.2) * np.sqrt(8),
                    (self.iy + 0.5 * self.Ly) * np.sqrt(8),
                    (self.iz + 0.5 * self.Lz) * np.sqrt(8),
                ],
                self.field,
                self.Lz * np.sqrt(8) / 2,
                0.3,
                0.06 * self.Lz,
                1.8,
            )

            x, y, z = zip(*field_direction.coordenadas)
            ax.plot_surface(np.array(x), np.array(y), np.array(z), color="navy")

        else:

            ax.quiver(
                (self.ix + self.Lx + 0.2) * np.sqrt(8),
                (self.iy + 0.5 * self.Ly) * np.sqrt(8),
                (self.iz + 0.5 * self.Lz) * np.sqrt(8),
                *self.field,
                length=self.Lz * np.sqrt(8) / 2,
                arrow_length_ratio=0.3,
                pivot="middle",
                capstyle="round",
                colors="navy",
                lw=3
            )

        # Limits and aspect
        ax.set_xlim(
            self.ix * np.sqrt(8),
            (self.ix + max(self.Lx, self.Ly, self.Lz)) * np.sqrt(8),
        )
        ax.set_ylim(
            self.iy * np.sqrt(8),
            (self.iy + max(self.Lx, self.Ly, self.Lz)) * np.sqrt(8),
        )
        ax.set_zlim(
            self.iz * np.sqrt(8),
            (self.iz + max(self.Lx, self.Ly, self.Lz)) * np.sqrt(8),
        )

        # ax.set_aspect("equal")
        ax.set_box_aspect((1, 1, 1))

        # Hide axis. No uso ax.axis('off') para que pueda poner nombre a los ejes cuando sólo dibujo la red.
        ax.w_xaxis.set_pane_color((1.0, 1.0, 1.0, 0.0))
        ax.w_yaxis.set_pane_color((1.0, 1.0, 1.0, 0.0))
        ax.w_zaxis.set_pane_color((1.0, 1.0, 1.0, 0.0))

        ax.w_xaxis.line.set_color((1.0, 1.0, 1.0, 0.0))
        ax.w_yaxis.line.set_color((1.0, 1.0, 1.0, 0.0))
        ax.w_zaxis.line.set_color((1.0, 1.0, 1.0, 0.0))

        ax.set_xticks([])
        ax.set_yticks([])
        ax.set_zticks([])
